
## React 16

Highlights of the update include...

1. Fragments
2. Error Boundaries
3. Portals
4. Reduced File Size
5. New Core Architecture (Fiber)

See https://reactjs.org/blog/2017/09/26/react-v16.0.html for more details

### No more `React.createClass`

As of version 16, `React.createClass` is no longer part of the core library. You can addon a package `create-react-class` to perform this legacy function but it is recommended to use the React Class Component structure instead, or you can also create stateless functional components.

-----

## aek-lib (v1.0.0)

We have stripped `React.createClass` from aek-lib. All components have been now been refactored as component classes or stateless functions

### ES6 Modules

We have refactored all code in aek-lib to use ES6 module syntax in favour of the previously adopted CommonJS form, and we advise that all developer code follows suit.

Instead of...

```javascript
const React = require("react");
```
We now write...
```javascript
import React from "react";
```
See https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Statements/import for more info.

One advantage of ES6 modules is that in addition (or instead of) a default export, modules can include additional exports.

```javascript
// my-module.js

export default class MyClass {}

export function myClassHelper() {}

export MY_CONSTANT = 12345;

```

```javascript
// main-module.js

import MyClass,{myClassHelper,MY_CONSTANT} from "./my-module";

...

```

With modules defined in this way, the exports are clearly separable and this has huge advantages in terms of optimisation as Webpack now provides "tree shaking" which identifies dead code and strips it from your production bundles.

We now take advantage of this in aek-lib by bundling all modules into a single index so all aek-lib functions and components can be imported from the same place

Instead of...

```javascript
const Button = require("-components/button");
const {Listview} = require("-components/listview");
const {fuse,asArray} = require("-aek/utils");
```

We write...

```javascript
import {Button,Listview,fuse,asArray} from "@ombiel/aek-lib";
```

If you have a long list of imports, I recommend writing each import on its own line. This helps with readability as well as in git diffs.

```javascript
import {
  Segment,
  ButtonGroup,
  Button,
  Listview,
  ListviewItem
  fuse,
  asArray,
} from "@ombiel/aek-lib";
```

Everything from aek-lib can be imported from this one module with one exception. SimpleStore and simple store plugins have their own index.

```javascript
import {
  SimpleStore,
  loggerPlugin,
  localStoragePlugin,
  windowSizePlugin,
} from "@ombiel/aek-lib/simple-store";

```

### Utils / Lodash

Another important change is the omission of Lodash from our utils library. Our boilerplate configurations include special Lodash loaders that allow for lodash tree shaking - only including the specific lodash functions that we use in our code without the need to bundle the whole library.

To take advantage of this, you must again use ES6 import syntax...

**Don't** ...
```javascript
import _ from "lodash";

_.isArray(myArray)
```

**Do** ...
```javascript
import {extend,clone,isArray} from "lodash";

isArray(myArray);
```

Note that non-lodash functions that were in aek-lib utils are now available from the main aek-lib index.

### Documentation

The aek-libs documentation has been updated to reflect all the new changes.

----


## Developer Build Tools

aek-buildtools has been abandoned in favour of separate config presets for each of the main 3rd party libraries (Webpack, Babel and ESlint) and a new module "aek-devserver" has been added as a standalone package for running a local dev server, removing the need to have a local instance of aek-cli in every project

### babel-preset-aek

Babel is responsible for transforming new and future proposed JS syntax into code that is compatible with today's browsers. babel-preset-aek provides configuration for Babel to include our recommended set of plugins. It also includes all required dependencies including babel-core.

Out of the box, babel-preset-aek includes everything listed here up to and including ES2017 https://babeljs.io/docs/en/plugins

In addition, we have react/jsx plugins, object-rest-spread and class-properties.

Notable syntax additions include...

#### async/await

Nice syntax sugar for managing Promises.
```javascript
async function foo() {
  await bar();
}
```
https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Statements/async_function

#### object-rest-spread

```javascript
let { x, y, ...z } = { x: 1, y: 2, a: 3, b: 4 };
console.log(x); // 1
console.log(y); // 2
console.log(z); // { a: 3, b: 4 }
```

#### class-properties

A big gotcha when moving from `React.createClass` to class components is that component methods are no longer automatically bound to the instance. As this is a very common scenario, this is the primary reason we have included this "experimental" syntax to the preset.

Instead of...

```javascript
class MyComponent extends React.Component {

  constructor() {
    this.onClick = this.onClick.bind(this);
  }

  onClick(ev) { ... }
}
```

we can use a class property with an arrow function...

```javascript
class MyComponent extends React.Component {

  onClick = (ev)=>{ ... }

}
```

#### Polyfills

Babel is mainly focussed on transforming newer syntax, but it also provides polyfills for deficiencies in older browsers. these include support for native prototype methods that are absent in older browsers (eg Array.prototype.from).

Using the latest babel and babel-preset-env, all necessary polyfills are bundled automatically based on our source code and a simple browser target string. The browser target string can be configured specifically for any project but the default defined in babel-preset-aek is...

```bash
last 2 versions, > 1%,ie > 10,Android >= 4,iOS >= 10
```
For a list of browsers included in this target see browserl.ist...

https://browserl.ist/?q=last+2+versions%2C+%3E+1%25%2Cie+%3E+10%2CAndroid+%3E%3D+4%2CiOS+%3E%3D+10

You can try changing the query at browserl.ist to get an idea of how it works. Obviously, the more inclusive the target, the more polyfills required and the larger your JS bundles so it's important to get a good balance.

To get an idea of what polyfills are being included, you can run your projects build analyzer

```bash
$ aek build:analyze
```

Polyfills can be found under `core-js/modules`

---

babel-preset-aek is a standard babel preset and can be used in combination with other presets and plugins. A developer is free to override or extend any configuration in package.json or in a .babelrc config file.


### aek-webpack

Webpack is responsible for packaging the modules we reference in our code into defined bundles that we then utilise in the browser. This does not only include javascript, but we can also bundle css, less, images, fonts and any other resource required. Webpack performs this task by using specified "loaders" for handling different types of file. In webpack configuration, we generally specify loaders by associating them with a file extension, or a filepath pattern.

Out of the box, aek-webpack includes the following mappings...

`*.js,*.jsx` - babel-loader
`*.css` - postcss-loader, css-loader, style-loader
`*.less` - less-loader and all above css loaders
`*.styl` - stylus-loader and all above css loaders
`*.scss` - sass-loader* and all above css loaders
`*.(woff|eot|otf|woff2|png|jpg|jpeg|gif|svg|ttf)` - file-loader

\* unlike less and stylus, sass-loader (if required) needs to be installed as an additional dependency in the project

#### babel-loader

You no longer have to use `ES6.js` or `jsx` extensions to perform Babel transforms - all js files are treated the same.

In aek-webpack, babel-loader is configured to use babel-preset-aek for configuration but should automatically adopt any project-specific config from package.json or .babelrc

### CSS Loaders

All css based files go through multiple loaders...

##### postcss-loader

Performs automatic vendor prefixing (`-webkit`, `-moz`, etc) as well as minification/compression using cssnano (production only).

##### style-loader

Automatically adds any imported css into your document by dynamically creating a `<style>` tag in the DOM. There is no need for any additional setup.

##### css-loader

By default, css-loader is configured to treat all css files as css-modules. It will parse the sourcecode and rename any classnames, providing better scoping by avoiding name collisions. The loader will then return a hash of classnames to the importing module.

```less
// mycss.less

.myClass {
  background-color: #000;
  a:hover {
    color: #900;
  }
}

```

```css
/* rendered css */

.myClass_1kX0CNgr {
  background-color: #000;
}
.myClass_1kX0CNgr a:hover {
  color: #900;
}

```
```javascript
// mycomponent.js

import myStyles from "./mycss.less";

export default function MyComponent(props) {
  return <div className={myStyles.myClass} />;
}

```

```markup
<!-- rendered html -->

<div class="myClass_1kX0CNgr" />

```

###### Global Styles

It is recommended to avoid "global" class names but if there is no reasonable alternative, you can inject css classes without them being renamed. There are 2 methods.

1. **Declare "global" selectors inline in your css**

```css
/* mycss.css */*

.myClass {
  background-color: #000;
}

:global(.myGlobalClass) {
  background-color: #900;
}

```
```css
/* rendered css */

.myClass_1kX0CNgr {
  background-color: #000;
}
.myGlobalClass {
  background-color: #900;
}

```

2. **Create a "global" stylesheet**

    Naming your css files with a global extension (`*.global.less`,`*.global.css`) the webpack config will pass your sourcecode through a slightly modified chain of loaders. Behaviour is the same, but your classnames will not be modified


#### file-loader

file-loader can be used to import other resources such as images. It doesn't do anything special but simply exports a URL for the required resource. The resource itself will be part of the final webpack build, represented by a separate file and a hashed name. Its good practice to use this instead of manually trying to manage the public assets.

Unfortunately as the actual location of the resource for AEK cannot be predetermined prior to deployment, you still have to combine the resulting URL with the `publicPath` function from aek-lib.

```javascript

import myImage from "./images/my-image.jpg";
import {publicPath} from "@ombiel/aek-lib";

return <img src={publicPath(myImage)} />;

```

#### progress-bar

aek-webpack includes a progress-bar plugin that renders a small indicator in the console displaying the progress of the current build

#### bundle-analyzer

aek-webpack also includes a bundle-analyzer which generates a web-based report of your build, breaking down the resulting bundle into its constituent modules and providing stats on file-size. This can be very useful if you need to identify unusually large bundles or wish to pursue further optimisations.

This feature is off by default but can be enabled using the env variable `CM_BUNDLE_ANALYZE=yes` - a convenient script has been added to the new boilerplate as well so a build and analysis can be performed with the command `aek build:analyze`

#### Build Environments

aek-webpack has 2 environment modes. One for development builds and one for production. The environment defaults to "development" and can be set to "production" using the `NODE_ENV` environment variable.

The resulting behaviour for each environment build should be the same, the development build is optimised for quick build times and includes source-maps,  while the production build is optimised for filesize, performance and includes minification and other optimisations.

#### Dependencies

aek-webpack includes all required dependencies in its package.json including webpack itself. All you need to install is `aek-webpack` and this will include all of these sub-dependencies

#### Customisation

aek-webpack exports a function which returns the config object for webpack to use. In order to customise, you simply import aek-webpack in your own config file and modify the returning object as required.

```javascript
// webpack.config.js

const aekWebpackConfig = require("@ombiel/aek-webpack");

module.exports = function() {

  const config = aekWebpackConfig();

  // customise `config` here
  // https://webpack.js.org/configuration/

  return config;
};


```
The above file is included in the new aek-boilerplate


### eslint-config-aek

Technically linting is non-essential but I'd urge anyone to use it. Not only does it help consistency in code style across teams, but it also drastically improves quality and helps instantly identify potential issues.

Eslint can be configured either in package.json or through a `.eslintrc` config file. It is pluggable so you can add specialised plugins and you can also extend other configurations with your own personalised rules.

eslint-config-aek offers a sensible set of rules built from Airbnb's configuration. This is a very popular set of rules that is widely considered a "standard".

In addition to standard rules, it also includes plugins and configuration for...

- **eslint-plugin-import** (verify module reference paths)
- **eslint-plugin-react** (identify common react gotchas)
- **eslint-eslint-plugin-jsx-a11y** (identify possible accessibility issues in react components)

Eslint configuration can be done in package.json or in a dedicated `.eslintrc` file. To use eslint-config-aek, simply use the extends option like so...

```javascript
{
  "eslintConfig": {
    "extends": "@ombiel/aek"
    // add any modifications here
  },
}
```
The eslint-config-aek includes all required dependecies (including eslint itself)

Some minor modifications have been made that differ from Airbnb config. These are largely for AEK specific customisation and to provide better compliance with older code.

#### Atom Linter

Install 2 packages in Atom - "linter" and "linter-eslint". These should automatically find and use your projects eslint installation with the above configuration. You should now have real-time feedback as you code. Changes to eslint and/or config sometimes require that you restart your Atom instance.

#### Command Line Use

Alternatively, you can use eslint at the command line. With Yarn globally installed and eslint locally installed in your project...

```bash
$ yarn run eslint src/client
```

This will lint all files found in the src/client directory and provide a report if it finds any issues.

For convenience 2 scripts have been added to the new aek-boilerplate

```bash
$ aek lint
```
Executes eslint on the src/client directory

```bash
$ aek lint:fix
```
Executes eslint on the src/client directory and automatically corrects and "fixable" issues.

### aek-devserver

The "runserver" command has been removed from aek-cli and moved into its own standalone package. aek-devserver has been created as a CLI tool built on the same framework as aek-cli. This means it can potentially be used as a globally installed package but its mainly designed as a local dependency for aek projects.

Essentially, aek-devserver runs a preconfigured instance of webpack-devserver with some additional middleware to provide:

1. ECT compilation
2. AEK screen rendering via the remote environment
3. Startup hijacking to modify menu options and other config for your dev environment
4. A proxy for any additional requests to the desired remote server.

It essentially works much like the runserver config in previous boilerplates but is preconfigured to use all the above middleware - you no longer need to add these as "plugins" in your runserver.yaml

#### runserver.yaml

runserver config has been simplified a bit in a couple of ways.

1. No need for standard plugin declarations

webpack-middleware, css-middleware, screen-middleware are no longer required and will be ignored in your config. You can continue to use other plugins if required.

2. Devserver now supports `screenName` property on menu options and `noLabel` is false by default

instead of...

```yaml
plugins:
  "/public/js": "@ombiel/aek-buildtools/webpack-middleware"
  "/public/css": "@ombiel/aek-buildtools/css-middleware"
  "/aek/c": "@ombiel/aek-buildtools/screen-middleware"

homescreen:
  replace:
    - code: myscreen
      desc: "My Screen"
      devPath: "/aek/c/myscreen"
      noLabel: false
```
you can now simply have...
```yaml
homescreen:
  replace:
    - screenName: myscreen
      desc: "My Screen"
```


## aek-cli

aek-cli has been translated by hand from CoffeeScript to javascript. This will hopefully help in terms of maintenance.

Throughout aek-cli, the interface has been improved with some better feedback and error handling.

In addition, there have been a few notable changes...

### Removed `runserver` command

See above `aek-devserver` - we no longer need a local version of aek-cl for every project.

### Embedded Yarn

Many of the tasks previously performed by NPM are now performed by Yarn. Yarn is generally more efficient and widely considered better in other ways. Recent versions of NPM have been much improved but have caused backwards compatibility issues.

You do not need to install Yarn globally as aek-cli will use its own embedded version. You can perform any yarn command with the embedded version using the `aek yarn` command but most common tasks have there own dedicated aek equivalent anyway.

-------

Below is a current list of commands and an explanation of their behaviour. These are not all new.

### `aek auth`

Contacts AppManager to renew a current auth token. If the user's current auth token is stale (15 mins lifespan) or the user has no previous auth token, the user is prompted for username and password.

Tokens are base64 encoded and stored in the users .npmrc as "passwords"

### `aek create`

Creates a new aek project based on a selected boilerplate. Once the boilerplate has been downloaded, it is modified slightly with any command properties and then an `aek install` is performed. If the boilerplate includes an `aek:postcreate` package script, this will be executed after the install.

### `aek deploy`

aek deploy behaves differently given the current status of the project. This has lead to some confusion. The solution we've come up with is designed to maintain current behaviour but also allow for a hook (`aek:prerelease`) to replace legacy behaviour in new projects.

The process flows a bit like this...

1. Checks if the current local version from package.json already exists in the registry. If it does and the user did not use the `-n` flag, it simply instructs AppManager to pull the existing package from the registry and publish to the specified host (see step 5)


2. If the command was executed with the `-n` flag or the version or package does not already exist in the registry, it is assumed that the project needs to be built before deploying and it needs to perform predeploy tasks. At this point, it checks package.json for a script named "aek:prerelease" (the new aek-boilerplate includes this - see below). If an "aek:prerelease" script exists, it is executed. If the script exits with a non 0 exit code, the whole process is aborted. If it exits with 0, then it moves straight to the push stage (step 4)


3. If an "aek:prerelease" script is not found, it defers to the old legacy behaviour.
- It checks for a gulp task named "\_predeploy" and runs it if found. A "\_predeploy" gulp task exists in the old boilerplate, it basically builds the project from source and outputs to the projects `.build` directory and adds a build commit in the git repo (if there is one).
- Once the \_predeploy task is complete, it then checks if the version is the same as the current "latest" version in the registry. If it is, the patch version is bumped with `npm version patch` (this will attempt to add a tag to the git repo)


4. Push Stage - At this stage, we should have a built project ready to deploy. The whole package is "pushed" to the registry. This is essentially an `aek push` which is basically a `yarn publish`.


5. Deploy Stage - Once the package has been pushed to the registry, a request is made to AppManager. AppManager will download the package tarball from the registry and extract the ".build" contents, each file found in `.build/public` is added to the static asset server (eg portal.ombiel.co.uk). Each ".twig" file in `.build/screens` becomes an aek screen - it is either created or overwrites an existing screen with the same package/screen name.


6. A list of screens and public assets that have been deployed are returned to the user in JSON format.

#### aek:prerelease

The aek:prerelease script can be any executable package script. There is a "standard" prerelease script found in aek-devserver that is included in the aek-boilerplate. The standard script can be replaced. It can also be used as part of a customised script if you wish to keep the core process but perform customised build steps before or after.

The standard aek:prerelease script works like this...

1. Check if the project is a git repo, if it is, ensure the repo is clean (no uncommitted changes). If the git repo is not clean the process is aborted


2. Check the current local package name and version in package.json


3. If the package/version already exists in the registry, prompt the user for a new version number.


4. The new version number entered by the user is checked against the registry and the user will be re-prompted until they provide a valid version number


5. Once established the version is updated in package.json (there is no git tag at this stage)


6. Builds the project, performs a production webpack build and translates ect to twig files. The build is output to ".build" directory


7. If the project is a git repo, it then attempts to commit the build. The commit message will be the new version number. This step fails gracefully - the user will be presented with a warning but the script will continue.


8. If the build was committed, we attempt to git tag the new commit. The tag name will be the version number. Again, this fails gracefully with a warning message. This avoids problems we've had internally with multi-branch projects and git tag conflicts.



### `aek info [name]`

Returns info from the registry about a specified package.

This is the equivalent of...

```bash
$ aek auth
$ yarn info @ombiel/[name]
```

### `aek install`

Install project dependencies. This is equivalent to

```bash
$ aek auth
$ yarn install
```

### `aek install [name]`

Included for consistency with legacy aek-cli

"Add" an aek package to your project. Equivalent to...

```bash
$ aek auth
$ yarn add @ombiel/[name]
```

### `aek logout`

Removes all cm user details (email, username, token) from the user's .npmrc

### `aek login`

Similar to `aek auth` but it always prompts the user for new credentials ignoring any existing token.

### `aek pull`

Pulls a package from the aek registry and installs it

### `aek push`

Pushes a package to the aek registry. This is mainly used for publishing library code - when you don't want to deploy.

Equivalent to...

```bash
$ aek auth
$ yarn publish
```

### `aek uninstall [name]`

Removes an aek package from your project

Equivalent to...

```bash
$ aek auth
$ yarn remove @ombiel/[name]
```

### `aek view-access`

Print aek permissions for the current project.

### `aek access`

Modify permissions for the current project.

### `aek yarn ...args`

Use embedded yarn cli

### `aek add [name]`

Add a dependency to your project. Equivalent to...

```bash
$ aek auth
$ yarn add [name]
```
Supports all `yarn add` options

### `aek remove [name]`

Remove a dependency from your project. Equivalent to...

```bash
$ aek auth
$ yarn remove [name]
```
Supports all `yarn remove` options

### `aek tag`

A wrapper around `yarn tag`. Similar in behaviour but with some aek style helpers so you can perform operations interactively. This is here to enable management of tags on the aek registry.

You can add a tag like this...

```bash
$ aek tag add my-package@1.0.0 mytag
```
This enables you to then reference that version by tag, so you could for example...
```bash
$ aek add my-package@mytag
```
Unless you are publishing library code, this is unlikely to be of much use but it can be critical if you need to manage the `latest` tag as this determines the default package when pulled with no version specified. It also determines what appears in the documentation on npm.campusm.net

Note that there is always a `latest` tag. If you remove it...

```bash
$ aek tag remove my-package latest
```
The server will automatically tag the version it determines to be the "latest" version.

Generally, every time you deploy a new build, that becomes `latest` automatically.

You can list the available tags for a package...

```bash
$ aek tag list my-package
```
You can also enter interactive mode by simply using...
```bash
$ aek tag
```

### `aek import`

Generates yarn.lock for your project if it was previously managed by package-lock.json.

See https://yarnpkg.com/en/docs/cli/import

### `aek upgrade`

Updates dependencies to their latest version (within the constraints defined by package.json)

Equivalent to...

```bash
$ aek auth
$ yarn upgrade
```

See https://yarnpkg.com/en/docs/cli/upgrade

### `aek upgrade-interactive`

Interactive version of `aek-upgrade` allows you to manually choose what to update.

Equivalent to...

```bash
$ aek auth
$ yarn upgrade
```
See https://yarnpkg.com/en/docs/cli/upgrade-interactive

### `aek run`

Run a defined package script

See https://yarnpkg.com/en/docs/cli/run

### Package Scripts

Package scripts are project specific scripts defined in package.json.

Package scripts can be used to hook into various points of the yarn/npm processes. Adding an `install` script, for example, will ensure that the defined script is executed after a `yarn install` has completed

There are also a number of name conventions for common project related tasks. `start` for example is generally used to start an app, `test` to run unit tests, etc

It has become common practice to use package scripts for other project-related tasks as well. This is a simpler approach to task management that used to be performed using gulp in the previous boilerplates.

In most cases, you can omit the `run` command and run the package script directly.

For example, instead of `aek run start` you can simply use `aek start` (as long as the script name does not conflict with any other command)

Package scripts are also listed in the aek-cli when in interactive mode. (ie `$ aek` with no arguments)


## aek-css

There is a branch of cm-lib (withStyles) that is already in development as part of the IaaP effort. The aim in IaaP is to develop and build generic screens that can are customised purely through predetermined configuration. We cannot afford for the configuration to be handled at build time and typically we want/need config to be applied at runtime. Part of configuration is styling and a particularly complex part of styling is colour (this is also the most likely part to differ from customer to customer)

This type of configuration is something we also have missed in AEK and hopefully, we can transfer the same principles and practice from IaaP to AEK in the future

As we need to perform this style configuration at runtime on the client, we cannot rely on the existing aek-css configuration solution as this is a slow, heavyweight process that we have to do at build time.

There is a growing trend of "css-in-js" solutions that offer good dynamic style handling and css management. Beyond our specific requirement, these offer additional advantages over traditional css style sheets. 2 popular libraries are [styled-components](https://www.styled-components.com/) and [emotion](https://emotion.sh/) - the latter being the solution currently employed in the withStyles branch.

When we move to a css-in-js solution its going to be important to make the coupling of aek-lib and aek-css much tighter and continuing to manage the packages independently will be impractical. Also, to back the notion that we should make AEK a bit more opinionated, we should not be encouraging the use of either of these libraries independently.

At some time in the near future, the withStyles branch will become cm-lib@2.0.0 and this will contain all the style code translated from aek-css into a more configurable form. As this effort is already well underway, we did not think it necessary to spend much time on aek-css in this update as it will likely become redundant.

To move this a little towards the end goal, however, we have coupled all the aek-css modules with their respective aek-lib counterparts. With some webpack config, the new aek-boilerplate is set up to implicitly bundle in all the required css from aek-css automatically based on the aek-lib components that you use in your project. Ultimately this means you no longer have to maintain a css file and this aspect has been removed from the boilerplate.

If you wish to configure the aek-css styles, there are options for primary and secondary colour that can be defined in package.json. aek-boilerplate comes with these properties predefined and you can also switch the "injectStyles" to `false` to prevent this behaviour altogether should you need to. The boilerplate config looks like this

```javascript
{
  "ombiel": {
    "settings": {
      "aek-lib": {
        "injectStyles": true,
        "styles": {
          "primaryColor": "#444444",
          "secondaryColor": "#6f8e00"
        }
      }
    }
  }
}

```

If you need to create additional customisations, this can be performed by overriding rules in css modules (see above css loaders) Alternatively, if necessary, you can switch off the injection and manage a separate stylesheet in the old way.


## aek-boilerplate

aek-boilerplate is the new "standard" boilerplate for aek. Aside from utilizing all the new aek packages, it is designed to be more lightweight, more "opinionated" but also sufficiently customisable and easy to configure.

The original boilerplates were designed with flexibility and transparency in mind but the general feeling is that most users are happy to keep things simple and follow convention. By removing some of the "boilerplate" code and making opinionated assumptions about how the project will be structured and managed, we can simplify things for the common use cases.

Assumptions we make...

1. Client source code will be written in JS (with Babel transforms)  


2. Developer will be using React with aek-lib and aek-css (implicitly)  


3. All assets (css, images, etc) will be bundled with webpack and referenced as ES6 `import`s in the JS code  


4. Each AEK screen will reference a single (similarly named) JS entry point (eg server/main.ect and client/main.js) and follow the structure defined below


5. AEK Screens will be defined using ECT templates and utilise a generic master template


6. Developers use an IDE that supports eslint (optional)

aek-boilerplate should include everything you need for a typical AEK project. The src directory has changed from js/css/screens to client/server where every JS file at the root of `client` will be a JS entry point and result in a JS file in the build output. Every ect file at the root of the `server` directory will be converted to Twig code and output as an "AEK Screen" in the resulting output. There is no need for a separate css directory as css is now processed with webpack alongside the JS code. The typical structure for a screen named "my-screen" might look like this.

```
- src
  |
  |- client
  |  |
  |  |- my-screen
  |  |  |
  |  |  |- styles
  |  |  |  - my-component.less
  |  |  |  - my-other-component.less
  |  |  |
  |  |  |- components
  |  |  |  - my-component.js
  |  |  |  - my-other-component.js
  |  |  |
  |  |  |- utils
  |  |     - my-utils.js
  |  |  
  |  |- my-screen.js
  |
  |- server
     |
     |- templates
     |  |
     |  |- master.ect
     |
     |- my-screen.ect
```

To help enforce this structure and encourage good naming, we no longer include a default "main" screen in the boilerplate. Instead, screens can be created using "generators". Generators can be executed from the command line. You will be asked for a few parameters and the templated scaffolding will be created for you.

When you first create a project with `aek create`, a screen generator will automatically be triggered to create an initial screen.

### Boilerplate Package Scripts

#### `aek start`

Start aek-devserver for the project

#### `aek build`

Build the current project - outputs to `.build` directory

#### `aek build:analyze`

Build the project as above and start bundle analyzer afterwards

#### `aek devserver`

Shortcut to aek-devserver - if you need to perform aek-devserver commands directly

#### `aek lint`

Run eslint on the src/client directory

#### `aek lint:fix`

Run eslint as above but with the `--fix` flag which automatically corrects any fixable issues

#### `aek gen:screen`

Triggers the screen generator to create a screen structure from code template

#### `aek aek:postcreate`

You never need to call this directly but its here as a hook for the project creation. Its called once a new project is created with the boilerplate. Currently, this is set to simply trigger the above `gen:screen` script

#### `aek aek:prerelease`

a standard prerelease script from aek-devserver. Again, it's rare you'll want to call this directly but its an important hook in the new deploy mechanics (see above)

## aek2-examples

Simon has ported over most of the examples from the original aek-examples package. He has updated them to use the new boilerplate and the latest package versions in this update.

aek2-examples is available on NPM.
