# Installation Details

The following instructions are to get your machine ready for using AEK CLI. This should take no more than 30 minutes for you to get up and running.

Most of the steps are the same for Windows and OSX. The biggest difference for Windows is that you will need to ensure installed programs are on your system PATH in order to be accessible. See [Installation Issues](15-installation-issues) for more details.

If you do not wish to take these steps on your PC, or you have issues or limitations on your environment, an alternative may be to use an [online cloud IDE](20-cloud-ide).

## App Manager Accounts

An App Manager account is required to use the AEK CLI. The authentication is the same as you would use logging into App Manager. 

As a number of tools are built on npm protocols that require an email address, you must ensure that your App Manager account has an **email address** associated with it. This can be managed in App Manager, under User/Permissions > Users.

## Additional Software

There are a few software prerequisites that you will need to install before we can get the AEK CLI installed if you don't already have them.

Below are the steps of installing this software.

### Git

Git is not a direct dependency of AEK CLI, but it may be required to download third party modules that are only available via Git.

Git is available from https://www.git-scm.com/downloads.

### Node.js

Node.js provides the Javascript runtime that most are tools run on.

You can download it [from here](https://nodejs.org/dist/latest-v14.x/), and it also provides convenient installers for MacOSX and Windows.

**Our recommended version of Node is 14**. This complements our existing React 16 boilerplates. Any version of Node 14 will do, though the latest is usually the best.

Please note that Node.js comes with an appropriate version of npm, so you shouldn't need to install them separately. You will encounter issues on other major versions for the time being.

**Please note** that if you have React 15 projects that you still need to maintain, you need to continue running these under Node 10 (again, any version will do, but the latest is recommended). The latest 10.x release [can be found here](https://nodejs.org/dist/latest-v10.x/).

### Yarn

Since the React 16 release, the AEK framework now uses Yarn (1.x) for package management and various other tasks (instead of npm). Install it via npm with the following command:

`npm install -g yarn`

## Pre-Flight Checks

You should now be able to run the following commands

* `git --version`
* `node --version`
* `npm --version`
* `yarn --version`

If you have any problems at this stage, please read **Installation Issues** on the [next page](15-installation-issues).

## Installing `aek-cli`

Firstly, use NPM config to link the @ombiel packages:
   
```bash
npm config set @ombiel:registry https://npm.campusm.net/
```

You might find on your terminal this might not work. You can check the existing configuration by using `npm config list`. If you don't see `@ombiel:registry = "https://npm.campusm.net/"` under user config, please try:

```bash
npm config set "@ombiel:registry" https://npm.campusm.net/
```

And test again with `npm config list`.

Once that's working, you can use Yarn to fetch the CLI itself:

   `yarn global add https://npm.campusm.net/get/aek-cli`

This will then download and install the AEK CLI on your machine.

Once complete you should be able to execute the command `aek` which should return a menu of AEK commands. If you have any problems, please continue to the [next page](15-installation-issues).

## Logging into the CLI

As mentioned above, you use your App Manager account to log into the AEK CLI. However this can vary depending on your geographical region.

Alternatively, you may wish to test on your Preview environment. This is a separate environment to the one to your Production and Sandbox apps run off of. Please remember to logout with `aek logout` before trying to switch environments - the token will fail silently in a number of use cases and it's not always obvious what has gone wrong.
 
To authenticate against a specific environment, you need the correct prefix. You specify an environment by putting it in front of your username, separated by two colons, when logging into the AEK CLI. For example: `na::joe_bloggs`. Here is a list of prefixes and their corresponding App Manager URLs:

* `eu::` - Europe - https://appmanager.ombiel.com
* `ap::` - APAC - https://appmanager-ap.campusm.exlibrisgroup.com
* `na::` - North America (excluding Canada) - https://appmanager-na.campusm.exlibrisgroup.com
* `ca::` - Canada - https://appmanager-ca.campusm.exlibrisgroup.com
* `pe::` - Europe (Preview) - https://appmanager-eu01-preview.campusm.exlibrisgroup.com
* `pn::` - APAC (Preview) - https://appmanager-na01-preview.campusm.exlibrisgroup.com
* `pa::` - North America (Preview, excluding Canada) - https://appmanager-ap01-preview.campusm.exlibrisgroup.com
* `pc::` - Canada (Preview) - https://appmanager-ca01-preview.campusm.exlibrisgroup.com

The CLI has an automatic timeout of 20 minutes of inactivity. This also applies when using App Manager.

**Please note** - you may find the Preview prefixes do not work on older React 16 and React 15 projects. The CLI will give you a URI-related error message. This is covered in **Installation Issues** on the [next page](15-installation-issues).
