# Installation Issues

## Issues with Yarn

### `aek` is not recognised

You might encounter issues with installing Yarn packages globally. If so, follow the steps below:

1. Check if your yarn prefix is set with `yarn config get prefix` (from the command line).

2. It should be set to the parent directory of `yarn global bin` (i.e. `../AppData/Local/Yarn`, *not* the bin folder itself).

3. If it's undefined, or set to a folder that looks incorrect, use `yarn config set prefix FILEPATH` to do change it (where `FILEPATH` is the directory path in question).

### Adding Yarn to your system PATH (Windows-only)

1. Add the actual bin folder (from `yarn global bin`) to your system PATH with `set PATH=%PATH%;FILEPATH`

2. If that doesn't work, also add the parent directory (i.e. `../AppData/Local/Yarn`) itself.

You can this through Windows Explorer, instead of through the command line. See:

`Computer -> Properties -> Advanced System Settings -> Environment Variables`

Or you can search for "Environment Variables" in the Start Menu.

## Issues with the CLI

### `aek` "Cannot read property 'substring' of undefined"

This is usually due to you attempting to call the AEK CLI from a bash shell (like Git Bash, or Cygwin) on Windows. This shouldn't be a problem on MacOS or a Linux distro. Please stick to Command Prompt, PowerShell or Terminal on Windows.

You may find after switching from bash that some CLI commands error trying to find `bash.exe`. You'll need to reset your Yarn and / or NPM config to clear any shell settings. See this [Github issue](https://github.com/yarnpkg/yarn/issues/5717#issuecomment-545681595) for details.

### Invalid URI "undefined/aek/auth/user-details"

This happens when you're logging in with an invalid environment prefix (see **Logging into the CLI** on the [previous page](10-installation-details)). The latest version of the CLI should have an up-to-date list of prefixes, but on older projects this isn't guaranteed.

You can however directly-modify the file these are stored in. This has to be done on a per-project basis, and the changes will be lost if you have to delete your node_modules folder (and run `aek install` again). The location of this file varies depending on the version of CLI used when creating the AEK project:

React 16 (with aek-cli-support): `node_modules/@ombiel/aek-cli-support/config.js`
React 16 (without aek-cli-support), React 15: `node_modules/@ombiel/aek-cli/src/config.coffee`

Here is a list of entries that should work as-is if copied and pasted into the `config.js` of `@ombiel/aek-cli-support`. The formatting may differ in older projects - please respect the formatting of the file you're editing (over the list here):

* Europe: `eu: "https://appmanager.ombiel.com",`
* APAC: `ap: "https://appmanager-ap.campusm.exlibrisgroup.com",`
* North America (excluding Canada): `na: "https://appmanager-na.campusm.exlibrisgroup.com,"`
* Canada: `ca: "https://appmanager-ca.campusm.exlibrisgroup.com",`
* Europe (Preview): `pe "https://appmanager-eu01-preview.campusm.exlibrisgroup.com",`
* APAC (Preview): `pn: "https://appmanager-na01-preview.campusm.exlibrisgroup.com",`
* North America (Preview, excluding Canada): `pa: "https://appmanager-ap01-preview.campusm.exlibrisgroup.com",`
* Canada (Preview): `pc: "https://appmanager-ca01-preview.campusm.exlibrisgroup.com",`

## Issues with other software

### Adding Node.js to your system PATH (Windows-only)

For installation issues with Node.js, you may need to take the additional step of adding Node to your system PATH.

Check if it's there, and if it's missing, add the appropriate Node folder path to the PATH variable, e.g.

`C:\apps\nodejs\`

If you're still getting an error saying that the command has not be found you may need to edit your PATH settings once again. Find the following in your PATH:

`%USERPROFILE%\AppData\Roaming\npm`

Change this to your Windows User, e.g.

`C:\Users\Simon\AppData\Roaming\npm`

Once this is done try `aek` again and this should now work.

### Adding Git to your system PATH (Windows-only)

Even with Git installed, you may need to ensure that it has been added to your system PATH.

If it's missing, add the the appropriate path to the PATH variable, e.g.

`C:\Program Files (x86)\Git\bin` or `C:\Program Files\Git\bin`
