#!/usr/bin/env node
const log = require("@ombiel/aek-cli-support/logger");
const {sortBy} = require("lodash");
const fs = require("fs");
const path = require("path");
const CLI = require("@ombiel/aek-cli-support/cli");


const {start: startNiceErrors, printNiceError} = require("@ombiel/aek-cli-support/nice-error");

const yarnProxy = require("./src/yarn-proxy");
const addPackageTasks = require("./src/add-package-tasks");

startNiceErrors();

let commands = [
  require("./src/commands/auth"),
  require("./src/commands/create"),
  require("./src/commands/deploy"),
  require("./src/commands/info"),
  require("./src/commands/install"),
  require("./src/commands/logout"),
  require("./src/commands/login"),
  require("./src/commands/pull"),
  require("./src/commands/push"),
  require("./src/commands/uninstall"),
  require("./src/commands/view-access"),
  require("./src/commands/access"),
  require("./src/commands/tag"),
  require("./src/commands/yarn"),
  yarnProxy("add", "add package dependency (see yarn add)"),
  yarnProxy("import", "Generates yarn.lock from package-lock.json (see yarn import)"),
  yarnProxy("remove", "remove package dependency (see yarn remove)"),
  yarnProxy("upgrade","Upgrades packages to their latest version (see yarn upgrade)"),
  yarnProxy("upgrade-interactive","Check for dependency updates (see yarn upgrade-interactive)"),
  yarnProxy("run","Check for dependency updates (see yarn run)",{noAuth: true}),
];

addPackageTasks(commands);

commands = sortBy(commands,"name");

const cli = new CLI(commands,{
  log: log,
  scriptName: "aek",
  getVersion: ()=>({
    "aek-cli": JSON.parse(fs.readFileSync(path.resolve(__dirname,"./package.json"))).version
  }),
});


cli.ready.then(function() {
  cli.parse();
})
.catch(function(e) {
  printNiceError(e);
});
