const fs = require("fs-extra");
const Path = require("path");
const {get,set,includes,remove} = require("lodash");
const chalk = require("chalk");

const {FriendlyError} = require("@ombiel/aek-cli-support/nice-error");

const printAccess = require("../print-access");

module.exports = {
  name: "access",
  help: "Add/Remove access to a package for a User/Host",
  options: [
    {
      name: "action",
      position: 1,
      help: "Type of action you would like to do. Add or Remove",
      choices: ["add","remove"],
      promptIfMissing: true
    },
    {
      name: "type",
      position: 2,
      help: "Type of permission you would like to set. Host or user",
      message: "Type of permission you would like to set:",
      choices: ["user","host"],
      promptIfMissing: true
    },
    {
      name: "name",
      position: 3,
      help: "Username or Hostname you wish to add",
      promptIfMissing: true
    },
    {
      name: "accessType",
      position: 4,
      help: "Which access type would like to set. Read or write",
      message: "Type of access you would like to set:",
      choices: ["read","write"],
      promptIfMissing: true
    }
  ],
  callback(options) {

    function commit(pkg,which) {

      return fs.writeFile(Path.resolve("./package.json"),JSON.stringify(pkg,null,2))
      .then(()=>{
        console.log([ //eslint-disable-line no-console
          "",
          chalk.green.underline(`${options.name} has been ${which} ${which === "added" ? "to" : "from"} ${options.type}s with the permissions of ${options.accessType}`),
          chalk.green.underline("To make these changes public please push this version to npm.campusm.net using aek push"),
          "",
        ].join("\n"));

        return printAccess();
      });

    }


    return fs.readJson(Path.resolve("./package.json"))
    .catch(()=>{
      throw new FriendlyError("No package.json found, are you in the directory of the AEK project?");
    })
    .then((pkgjson)=>{

      let list;

      const access = get(pkgjson,"ombiel.access");

      const {accessType,type,action,name} = options;

      if (action === "add") {
        list = get(access,[accessType,`${type}s`],[]);
        if (includes(list,name)) {
          console.log([ //eslint-disable-line no-console
            "\n",
            chalk.yellow.underline(`${name} is already added to ${type}s with the permissions of ${accessType}`),
            "\n",
          ].join(""));
          process.exit();
        }
        list.push(name);
        set(access,[accessType,`${type}s`],list);
        set(pkgjson,"ombiel.access",access);
        return commit(pkgjson,"added");
      }

      list = get(access,[accessType,`${type}s`],[]);
      if (includes(list,name)) {
        remove(list, item=> item === options.name);
        set(pkgjson,"ombiel.access",access);
        return commit(pkgjson,"removed");
      }

      console.log([ //eslint-disable-line no-console
        "\n",
        chalk.yellow.underline(`Cannot remove ${options.name} from ${options.type}s as the access type of ${options.accessType} does not exist`),
        "\n",
      ].join(""));

      return process.exit();


    });


  }
};
