const Path = require("path");
const fs = require("fs-extra");

const auth = require("@ombiel/aek-cli-support/auth");
const yarn = require("@ombiel/aek-cli-support/yarn");
const {FriendlyError} = require("@ombiel/aek-cli-support/nice-error");


module.exports = {
  name: "push",
  help: "Push a package to the AEK registry.",

  options: [
    {
      name: "username",
      message: "Please enter your App Manager username",
      noPrompt: true,
    },
    {
      name: "password",
      message: "Please enter your App Manager password",
      inputType: "password",
      noPrompt: true,
    },
    {
      name: "tag",
      abbr: "t",
      message: "Registers the published package with the given tag",
      noPrompt: true,
    },
  ],

  callback: function(options) {

    return fs.readJSON(Path.resolve("./package.json"))
    .then((pjson)=>{
      // check name
      const name = pjson.name;
      if (!name) {
        throw FriendlyError('Package must have a name (eg {"name":"@ombiel/mypackage"})');
      }

      if (name.indexOf("@ombiel") !== 0) {
        throw FriendlyError('To push packages to the AEK registry, you must use the "ombiel" scope - please prefix your package name with "@ombiel" (eg {"name":"@ombiel/mypackage"})');
      }

      return auth.getUserDetails(options)
      .then(function() {
        const opts = {};
        if (options.tag) {
          opts.tag = options.tag;
        }
        return yarn.command("publish",opts);
      });
    });

  },
};
