const Path = require("path");
const fs = require("fs-extra");
const {includes} = require("lodash");

const auth = require("@ombiel/aek-cli-support/auth");
const yarn = require("@ombiel/aek-cli-support/yarn");


module.exports = {

  name: "tag",
  help: `manage registry tags for packages`,
  options: [
    {
      name: "username",
      message: "Please enter your App Manager username",
      noPrompt: true,
    },
    {
      name: "password",
      message: "Please enter your App Manager password",
      inputType: "password",
      noPrompt: true,
    },
    {
      name: "task",
      position: 1,
      help: "name of task you want to perform",
      message: "Select the task you wish to perform",
      choices: ["list","add","remove"],
      required: true,
      promptIfMissing: true,
    },
    {
      name: "packageName",
      position: 2,
      help: "name of package to perform task on",
      message: "Please enter the name of the package",
      promptIfMissing: true,
      required: true,
      beforeInquiry(opt) {
        return fs.readJSON(Path.resolve("./package.json"))
        .then((pkg)=>{
          opt.default = pkg.name;
        })
        .catch(()=>{})
        .then(()=>opt);
      },
    },
    {
      name: "tag",
      position: 3,
      help: "tag to add/remove",
      message: (values)=>`Please enter the name of the tag you wish to ${values.task}`,
      promptIfMissing: (values)=>{
        return includes(["remove","add"],values.task);
      },
    },
    {
      name: "version",
      help: "package version to apply the tag to",
      message: "Please specify the version to apply the tag to",
      promptIfMissing: (values)=>values.task === "add" && values.packageName.indexOf("@",1) === -1,
      beforeInquiry(opt) {
        return fs.readJSON(Path.resolve("./package.json"))
        .then((pkg)=>{
          opt.default = pkg.version;
        })
        .catch(()=>{})
        .then(()=>opt);
      },
    }
  ],
  callback: function(options) {

    return auth.getUserDetails(options)
    .then(function(/*userDetails*/) {

      let {packageName} = options;

      if (packageName.indexOf("@ombiel") !== 0) {
        packageName = `@ombiel/${packageName}`;
      }

      if (options.task === "add" && options.version && packageName.indexOf("@",1) === -1) {
        packageName = `${packageName}@${options.version}`;
      }

      const yarnArgs = [options.task,packageName];

      if (includes(["add","remove"],options.task) && options.tag) {
        yarnArgs.push(options.tag);
      }

      return yarn.command("tag",yarnArgs)
      .then(()=>{
        yarn.command("tag",["list",packageName.split("@").slice(0,2).join("@")]);
      });

    });
  },
};
