const fs = require("fs-extra");
const Path = require("path");
const chalk = require("chalk");
const {get,forEach,isArray} = require("lodash");

const {FriendlyError} = require("@ombiel/aek-cli-support/nice-error");

function hasLength(arr) {
  return isArray(arr) && (arr.length > 0);
}

module.exports = function printAccess() {

  return fs.readJson(Path.resolve("./package.json"))
  .catch(()=>{
    throw new FriendlyError("No package.json found, are you in the directory of the AEK project?");
  })
  .then((pkgjson)=>{

    const pkgName = get(pkgjson,"name");
    const access = get(pkgjson,"ombiel.access");
    let wusers = ['No users'];
    let wsections = ['No hosts'];
    let rusers = ['No users'];
    let rsections = ['No hosts'];
    forEach(access,(val,section)=>{
      if (val === true) {
        console.log([ // eslint-disable-line no-console
          `\n${chalk.yellow.underline(`Permissions set for ${pkgName}`)}`,
          "Package is set to public (read only)"
        ].join("\n\n"));

        process.exit();
      }
      else if (section === "write") {
        wusers = access[section].users;
        wsections = access[section].hosts;
        if (!hasLength(wusers)) {
          wusers = ['No users'];
        }
        if (!hasLength(wsections)) {
          wsections = ['No hosts'];
        }
      }
      else {
        rusers = access[section].users;
        rsections = access[section].hosts;
        if (!hasLength(rusers)) {
          rusers = ['No users'];
        }
        if (!hasLength(rsections)) {
          rsections = ['No hosts'];
        }
      }
    });


    return console.log([ // eslint-disable-line no-console
      `\n${chalk.yellow.underline(`Permissions set for ${pkgName}`)}`,
      `${chalk.green.underline("Users with write access")}\n${wusers.join("\n")}`,
      `${chalk.green.underline("Hosts with write access")}\n${wsections.join("\n")}`,
      `${chalk.green.underline("Users with read access")}\n${rusers.join("\n")}`,
      `${chalk.green.underline("Hosts with read access")}\n${rsections.join("\n")}\n`,
    ].join("\n\n"));

  });
};
