const Path = require("path");
const fs = require("fs-extra");

const auth = require("@ombiel/aek-cli-support/auth");
const npm = require("@ombiel/aek-cli-support/npm");
const request = require("request");
const tar = require("tar-fs");
const {get} = require("lodash");

module.exports = function(options) {

  if (options.package.indexOf("@workspace") === 0) {
    return fs.copy(Path.resolve(options.package.split("/")[1]),Path.resolve(options.destination),{
      filter(src) {
        if (src.indexOf(".git") > -1) {
          return false;
        }
        if (src.indexOf("node_modules") > -1) {
          return false;
        }
        return true;
      }
    });
  }

  return auth.getUserDetails(options)
  .then(function(userDetails) {
    return new Promise(function(resolve,reject) {
      const packageName = options.package;
      npm.view(packageName)
      .then(function(data) {

        if (!data) {
          reject(Error("Can't obtain package info"));
        }
        if (!get(data,"dist.tarball")) {
          reject(Error("Package tarball not available"));
        }

        const requestOptions = {
          url: data.dist.tarball,
          headers: {
            "aek-username": userDetails.username,
            "aek-token": userDetails.token,
          },
        };

        const destination = options.destination;

        const extractor = tar.extract(destination,{
          map: function(header) {
            header.name = header.name.replace(/^package[/\\]/,"");
            return header;
          }
        })
        .on('error', reject)
        .on('finish',resolve);

        request(requestOptions)
        .on("error",reject)
        .pipe(require('zlib').createGunzip())
        .pipe(extractor);

      })
      .catch(reject);
    });
  });
};
